@@ -16,15 +16,10 @@
 #include "toolcontext.h"
 #include "segtype.h"
 #include "display.h"
-#include "text_export.h"
-#include "text_import.h"
 #include "config.h"
 #include "str_list.h"
-#include "targets.h"
-#include "lvm-string.h"
 #include "activate.h"
 #include "str_list.h"
-#include "metadata.h"
 
 static const char *_errseg_name(const struct lv_segment *seg)
 {
@@ -40,20 +35,21 @@ static int _errseg_merge_segments(struct lv_segment *seg1, struct lv_segment *se
 }
 
 #ifdef DEVMAPPER_SUPPORT
-static int _errseg_add_target_line(struct dev_manager *dm __attribute((unused)),
-				struct dm_pool *mem __attribute((unused)),
-				struct cmd_context *cmd __attribute((unused)),
-				void **target_state __attribute((unused)),
-				struct lv_segment *seg __attribute((unused)),
+static int _errseg_add_target_line(struct dev_manager *dm __attribute__((unused)),
+				struct dm_pool *mem __attribute__((unused)),
+				struct cmd_context *cmd __attribute__((unused)),
+				void **target_state __attribute__((unused)),
+				struct lv_segment *seg __attribute__((unused)),
+				const struct lv_activate_opts *laopts __attribute__((unused)),
 				struct dm_tree_node *node, uint64_t len,
-				uint32_t *pvmove_mirror_count __attribute((unused)))
+				uint32_t *pvmove_mirror_count __attribute__((unused)))
 {
 	return dm_tree_node_add_error_target(node, len);
 }
 
 static int _errseg_target_present(struct cmd_context *cmd,
-				  const struct lv_segment *seg __attribute((unused)),
-				  unsigned *attributes __attribute((unused)))
+				  const struct lv_segment *seg __attribute__((unused)),
+				  unsigned *attributes __attribute__((unused)))
 {
 	static int _errseg_checked = 0;
 	static int _errseg_present = 0;
@@ -70,7 +66,7 @@ static int _errseg_target_present(struct cmd_context *cmd,
 #endif
 
 static int _errseg_modules_needed(struct dm_pool *mem,
-				  const struct lv_segment *seg __attribute((unused)),
+				  const struct lv_segment *seg __attribute__((unused)),
 				  struct dm_list *modules)
 {
 	if (!str_list_add(mem, modules, "error")) {
@@ -81,9 +77,9 @@ static int _errseg_modules_needed(struct dm_pool *mem,
 	return 1;
 }
 
-static void _errseg_destroy(const struct segment_type *segtype)
+static void _errseg_destroy(struct segment_type *segtype)
 {
-	dm_free((void *)segtype);
+	dm_free(segtype);
 }
 
 static struct segtype_handler _error_ops = {
@@ -99,7 +95,7 @@ static struct segtype_handler _error_ops = {
 
 struct segment_type *init_error_segtype(struct cmd_context *cmd)
 {
-	struct segment_type *segtype = dm_malloc(sizeof(*segtype));
+	struct segment_type *segtype = dm_zalloc(sizeof(*segtype));
 
 	if (!segtype)
 		return_NULL;
