@@ -42,55 +42,59 @@
 /* Various flags */
 /* Note that the bits no longer necessarily correspond to LVM1 disk format */
 
-#define PARTIAL_VG		0x00000001U	/* VG */
-#define EXPORTED_VG          	0x00000002U	/* VG PV */
-#define RESIZEABLE_VG        	0x00000004U	/* VG */
-
-/*
- * Since the RAID flags are LV (and seg) only and the above three
- * are VG/PV only, these flags are reused.
- */
-#define RAID                    0x00000001U	/* LV */
-#define RAID_META               0x00000002U	/* LV */
-#define RAID_IMAGE              0x00000004U	/* LV */
+#define PARTIAL_VG		UINT64_C(0x00000001)	/* VG */
+#define EXPORTED_VG		UINT64_C(0x00000002)	/* VG PV */
+#define RESIZEABLE_VG		UINT64_C(0x00000004)	/* VG */
 
 /* May any free extents on this PV be used or must they be left free? */
-#define ALLOCATABLE_PV         	0x00000008U	/* PV */
+#define ALLOCATABLE_PV		UINT64_C(0x00000008)	/* PV */
 
-//#define SPINDOWN_LV          	0x00000010U	/* LV */
-//#define BADBLOCK_ON       	0x00000020U	/* LV */
-#define VISIBLE_LV		0x00000040U	/* LV */
-#define FIXED_MINOR		0x00000080U	/* LV */
+//#define SPINDOWN_LV		UINT64_C(0x00000010)	/* LV */
+//#define BADBLOCK_ON		UINT64_C(0x00000020)	/* LV */
+#define VISIBLE_LV		UINT64_C(0x00000040)	/* LV */
+#define FIXED_MINOR		UINT64_C(0x00000080)	/* LV */
 /* FIXME Remove when metadata restructuring is completed */
-#define SNAPSHOT		0x00001000U	/* LV - internal use only */
-#define PVMOVE			0x00002000U	/* VG LV SEG */
-#define LOCKED			0x00004000U	/* LV */
-#define MIRRORED		0x00008000U	/* LV - internal use only */
-//#define VIRTUAL			0x00010000U	/* LV - internal use only */
-#define MIRROR_LOG		0x00020000U	/* LV */
-#define MIRROR_IMAGE		0x00040000U	/* LV */
-#define LV_NOTSYNCED		0x00080000U	/* LV */
-//#define PRECOMMITTED		0x00200000U	/* VG - internal use only */
-#define CONVERTING		0x00400000U	/* LV */
+#define SNAPSHOT		UINT64_C(0x00001000)	/* LV - internal use only */
+#define PVMOVE			UINT64_C(0x00002000)	/* VG LV SEG */
+#define LOCKED			UINT64_C(0x00004000)	/* LV */
+#define MIRRORED		UINT64_C(0x00008000)	/* LV - internal use only */
+//#define VIRTUAL		UINT64_C(0x00010000)	/* LV - internal use only */
+#define MIRROR_LOG		UINT64_C(0x00020000)	/* LV */
+#define MIRROR_IMAGE		UINT64_C(0x00040000)	/* LV */
 
-#define MISSING_PV              0x00800000U	/* PV */
-#define PARTIAL_LV              0x01000000U	/* LV - derived flag, not
-						   written out in metadata*/
+#define LV_NOTSYNCED		UINT64_C(0x00080000)	/* LV */
+#define LV_REBUILD		UINT64_C(0x00100000)	/* LV */
+//#define PRECOMMITTED		UINT64_C(0x00200000)	/* VG - internal use only */
+#define CONVERTING		UINT64_C(0x00400000)	/* LV */
 
-//#define POSTORDER_FLAG	0x02000000U /* Not real flags, reserved for
-//#define POSTORDER_OPEN_FLAG	0x04000000U    temporary use inside vg_read_internal. */
-//#define VIRTUAL_ORIGIN	0x08000000U	/* LV - internal use only */
+#define MISSING_PV		UINT64_C(0x00800000)	/* PV */
+#define PARTIAL_LV		UINT64_C(0x01000000)	/* LV - derived flag, not
+							   written out in metadata*/
 
-#define MERGING			0x10000000U	/* LV SEG */
+//#define POSTORDER_FLAG	UINT64_C(0x02000000) /* Not real flags, reserved for
+//#define POSTORDER_OPEN_FLAG	UINT64_C(0x04000000)    temporary use inside vg_read_internal. */
+//#define VIRTUAL_ORIGIN	UINT64_C(0x08000000)	/* LV - internal use only */
 
-#define REPLICATOR		0x20000000U	/* LV -internal use only for replicator */
-#define REPLICATOR_LOG		0x40000000U	/* LV -internal use only for replicator-dev */
-#define UNLABELLED_PV           0x80000000U     /* PV -this PV had no label written yet */
+#define MERGING			UINT64_C(0x10000000)	/* LV SEG */
 
-#define LVM_READ              	0x00000100U	/* LV VG */
-#define LVM_WRITE             	0x00000200U	/* LV VG */
-#define CLUSTERED         	0x00000400U	/* VG */
-//#define SHARED            	0x00000800U	/* VG */
+#define REPLICATOR		UINT64_C(0x20000000)	/* LV -internal use only for replicator */
+#define REPLICATOR_LOG		UINT64_C(0x40000000)	/* LV -internal use only for replicator-dev */
+#define UNLABELLED_PV		UINT64_C(0x80000000)	/* PV -this PV had no label written yet */
+
+#define RAID			UINT64_C(0x0000000100000000)	/* LV */
+#define RAID_META		UINT64_C(0x0000000200000000)	/* LV */
+#define RAID_IMAGE		UINT64_C(0x0000000400000000)	/* LV */
+
+#define THIN_VOLUME		UINT64_C(0x0000001000000000)	/* LV */
+#define THIN_POOL		UINT64_C(0x0000002000000000)	/* LV */
+#define THIN_POOL_DATA		UINT64_C(0x0000004000000000)	/* LV */
+#define THIN_POOL_METADATA	UINT64_C(0x0000008000000000)	/* LV */
+
+#define LVM_READ		UINT64_C(0x00000100)	/* LV, VG */
+#define LVM_WRITE		UINT64_C(0x00000200)	/* LV, VG */
+
+#define CLUSTERED		UINT64_C(0x00000400)	/* VG */
+//#define SHARED		UINT64_C(0x00000800)	/* VG */
 
 /* Format features flags */
 #define FMT_SEGMENTS		0x00000001U	/* Arbitrary segment params? */
@@ -112,10 +116,10 @@
 /* vg_read and vg_read_for_update flags */
 #define READ_ALLOW_INCONSISTENT	0x00010000U
 #define READ_ALLOW_EXPORTED	0x00020000U
-#define READ_WITHOUT_LOCK       0x00040000U
+#define READ_WITHOUT_LOCK	0x00040000U
 
 /* A meta-flag, useful with toollib for_each_* functions. */
-#define READ_FOR_UPDATE 	0x00100000U
+#define READ_FOR_UPDATE		0x00100000U
 
 /* vg's "read_status" field */
 #define FAILED_INCONSISTENT	0x00000001U
@@ -132,6 +136,20 @@
 #define VGMETADATACOPIES_ALL UINT32_MAX
 #define VGMETADATACOPIES_UNMANAGED 0
 
+#define lv_is_thin_volume(lv)	((lv)->status & THIN_VOLUME ? 1 : 0)
+#define lv_is_thin_pool(lv)	((lv)->status & THIN_POOL ? 1 : 0)
+#define lv_is_used_thin_pool(lv)	(lv_is_thin_pool(lv) && !dm_list_empty(&(lv)->segs_using_this_lv))
+#define lv_is_thin_pool_data(lv)	((lv)->status & THIN_POOL_DATA ? 1 : 0)
+#define lv_is_thin_pool_metadata(lv)	((lv)->status & THIN_POOL_METADATA ? 1 : 0)
+#define lv_is_mirrored(lv)	((lv)->status & MIRRORED ? 1 : 0)
+#define lv_is_rlog(lv)		((lv)->status & REPLICATOR_LOG ? 1 : 0)
+
+#define lv_is_thin_type(lv)	((lv)->status & (THIN_POOL | THIN_VOLUME | THIN_POOL_DATA | THIN_POOL_METADATA) ? 1 : 0)
+#define lv_is_mirror_type(lv)	((lv)->status & (MIRROR_LOG | MIRROR_IMAGE | MIRRORED | PVMOVE) ? 1 : 0)
+#define lv_is_raid_type(lv)	((lv)->status & (RAID | RAID_IMAGE | RAID_META))
+
+#define lv_is_virtual(lv)	((lv)->status & VIRTUAL)
+
 /* Ordered list - see lv_manip.c */
 typedef enum {
 	AREA_UNASSIGNED,
@@ -148,6 +166,12 @@ typedef enum {
 	DONT_PROMPT_OVERRIDE = 2 /* Skip prompt + override a second condition */
 } force_t;
 
+typedef enum {
+	THIN_DISCARDS_IGNORE,
+	THIN_DISCARDS_NO_PASSDOWN,
+	THIN_DISCARDS_PASSDOWN,
+} thin_discards_t;
+
 struct cmd_context;
 struct format_handler;
 struct labeller;
@@ -156,10 +180,12 @@ struct format_type {
 	struct dm_list list;
 	struct cmd_context *cmd;
 	struct format_handler *ops;
+	struct dm_list mda_ops; /* List of permissible mda ops. */
 	struct labeller *labeller;
 	const char *name;
 	const char *alias;
 	const char *orphan_vg_name;
+	struct volume_group *orphan_vg; /* Only one ever exists. */
 	uint32_t features;
 	void *library;
 	void *private;
@@ -180,28 +206,26 @@ struct pv_segment {
 #define pvseg_is_allocated(pvseg) ((pvseg)->lvseg)
 
 /*
- * These flags define the type of the format instance to be created.
- * There are two basic types: a PV-based and a VG-based format instance.
- * We can further control the format_instance initialisation and functionality
- * by using the other flags. Today, the primary role of the format_instance
- * is to temporarily store metadata area information we are working with. More
- * flags can be defined to cover even more functionality in the future...
+ * Properties of each format instance type.
+ * The primary role of the format_instance is to temporarily store metadata
+ * area information we are working with.
  */
 
-/* PV-based format instance */
-#define FMT_INSTANCE_PV 		0x00000000U
+/* Include any existing PV ("on-disk") mdas during format_instance initialisation. */
+#define FMT_INSTANCE_MDAS		0x00000002U
 
-/* VG-based format instance */
-#define FMT_INSTANCE_VG 		0x00000001U
+/*
+ * Include any auxiliary mdas during format_instance intialisation.
+ * Currently, this includes metadata areas as defined by
+ * metadata/dirs and metadata/raws setting.
+ */
+#define FMT_INSTANCE_AUX_MDAS		0x00000004U
 
-/* Include any existing PV mdas during format_instance initialisation */
-#define FMT_INSTANCE_MDAS 		0x00000002U
-
-/* Include any auxiliary mdas during format_instance intialisation */
-#define FMT_INSTANCE_AUX_MDAS 		0x00000004U
-
-/* Include any other format-specific mdas during format_instance initialisation */
-#define FMT_INSTANCE_PRIVATE_MDAS 	0x00000008U
+/*
+ * Include any other format-specific mdas during format_instance initialisation.
+ * For example metadata areas used during backup/restore/archive handling.
+ */
+#define FMT_INSTANCE_PRIVATE_MDAS	0x00000008U
 
 struct format_instance {
 	unsigned ref_count;	/* Refs to this fid from VG and PV structs */
@@ -219,10 +243,7 @@ struct format_instance {
 	/* FIXME: Try to use the index only. Remove these lists. */
 	struct dm_list metadata_areas_in_use;
 	struct dm_list metadata_areas_ignored;
-	union {
-		struct metadata_area **array;
-		struct dm_hash_table *hash;
-	} metadata_areas_index;
+	struct dm_hash_table *metadata_areas_index;
 
 	void *private;
 };
@@ -241,6 +262,15 @@ struct lv_segment_area {
 	} u;
 };
 
+struct lv_thin_message {
+	struct dm_list list;		/* Chained list of messages */
+	dm_thin_message_t type;		/* Use dm thin message datatype */
+	union {
+		struct logical_volume *lv; /* For: create_thin, create_snap, trim */
+		uint32_t delete_id;	/* For delete, needs device_id */
+	} u;
+};
+
 struct segment_type;
 
 /* List with vg_name, vgid and flags */
@@ -265,9 +295,9 @@ struct replicator_site {
 
 	struct logical_volume *replicator; /* Reference to replicator */
 
-	const char *name;               /* Site name */
+	const char *name;		/* Site name */
 	const char *vg_name;		/* VG name */
-	struct volume_group *vg;        /* resolved vg  (activate/deactive) */
+	struct volume_group *vg;	/* resolved vg  (activate/deactive) */
 	unsigned site_index;
 	replicator_state_t state;	/* Active or pasive state of site */
 	dm_replicator_mode_t op_mode;	/* Operation mode sync or async fail|warn|drop|stall */
@@ -301,11 +331,12 @@ struct lv_segment {
 	uint64_t status;
 
 	/* FIXME Fields depend on segment type */
-	uint32_t stripe_size;   /* For stripe and RAID - in sectors */
+	uint32_t stripe_size;	/* For stripe and RAID - in sectors */
 	uint32_t area_count;
 	uint32_t area_len;
-	uint32_t chunk_size;	/* For snapshots - in sectors */
-	struct logical_volume *origin;
+	uint32_t chunk_size;	/* For snapshots/thin_pool.  In sectors. */
+				/* For thin_pool, 128..2097152. */
+	struct logical_volume *origin;	/* snap and thin */
 	struct logical_volume *cow;
 	struct dm_list origin_list;
 	uint32_t region_size;	/* For mirrors, replicators - in sectors */
@@ -317,7 +348,15 @@ struct lv_segment {
 	struct dm_list tags;
 
 	struct lv_segment_area *areas;
-	struct lv_segment_area *meta_areas; /* For RAID */
+	struct lv_segment_area *meta_areas;	/* For RAID */
+	struct logical_volume *metadata_lv;	/* For thin_pool */
+	uint64_t transaction_id;		/* For thin_pool, thin */
+	uint64_t low_water_mark;		/* For thin_pool */
+	unsigned zero_new_blocks;		/* For thin_pool */
+	thin_discards_t discards;		/* For thin_pool */
+	struct dm_list thin_messages;		/* For thin_pool */
+	struct logical_volume *pool_lv;		/* For thin */
+	uint32_t device_id;			/* For thin, 24bit */
 
 	struct logical_volume *replicator;/* For replicator-devs - link to replicator LV */
 	struct logical_volume *rlog_lv;	/* For replicators */
@@ -380,9 +419,9 @@ void pvcreate_params_set_defaults(struct pvcreate_params *pp);
 */
 int vg_write(struct volume_group *vg);
 int vg_commit(struct volume_group *vg);
-int vg_revert(struct volume_group *vg);
+void vg_revert(struct volume_group *vg);
 struct volume_group *vg_read_internal(struct cmd_context *cmd, const char *vg_name,
-			     const char *vgid, int warnings, int *consistent);
+				      const char *vgid, int warnings, int *consistent);
 struct physical_volume *pv_read(struct cmd_context *cmd, const char *pv_name,
 				int warnings,
 				int scan_label_only);
@@ -419,9 +458,9 @@ uint32_t vg_lock_newname(struct cmd_context *cmd, const char *vgname);
  * Return a handle to VG metadata.
  */
 struct volume_group *vg_read(struct cmd_context *cmd, const char *vg_name,
-              const char *vgid, uint32_t flags);
+			     const char *vgid, uint32_t flags);
 struct volume_group *vg_read_for_update(struct cmd_context *cmd, const char *vg_name,
-                         const char *vgid, uint32_t flags);
+			 const char *vgid, uint32_t flags);
 
 /* 
  * Test validity of a VG handle.
@@ -440,11 +479,11 @@ struct physical_volume *pv_create(const struct cmd_context *cmd,
 				  uint32_t existing_extent_count,
 				  uint32_t existing_extent_size,
 				  uint64_t label_sector,
-				  int pvmetadatacopies,
+				  unsigned pvmetadatacopies,
 				  uint64_t pvmetadatasize,
 				  unsigned metadataignore);
 int pv_resize(struct physical_volume *pv, struct volume_group *vg,
-             uint64_t size);
+	      uint64_t size);
 int pv_analyze(struct cmd_context *cmd, const char *pv_name,
 	       uint64_t label_sector);
 
@@ -489,7 +528,7 @@ struct logical_volume *lv_create_empty(const char *name,
 
 /* Write out LV contents */
 int set_lv(struct cmd_context *cmd, struct logical_volume *lv,
-           uint64_t sectors, int value);
+	   uint64_t sectors, int value);
 
 int lv_change_tag(struct logical_volume *lv, const char *tag, int add_tag);
 
@@ -507,7 +546,7 @@ int lv_extend(struct logical_volume *lv,
 	      const struct segment_type *segtype,
 	      uint32_t stripes, uint32_t stripe_size,
 	      uint32_t mirrors, uint32_t region_size,
-	      uint32_t extents,
+	      uint32_t extents, const char *thin_pool_name,
 	      struct dm_list *allocatable_pvs, alloc_policy_t alloc);
 
 /* lv must be part of lv->vg->lvs */
@@ -521,34 +560,44 @@ int lv_remove_with_dependencies(struct cmd_context *cmd, struct logical_volume *
 
 int lv_rename(struct cmd_context *cmd, struct logical_volume *lv,
 	      const char *new_name);
+int lv_rename_update(struct cmd_context *cmd, struct logical_volume *lv,
+		     const char *new_name, int update_mda);
 
 uint64_t extents_from_size(struct cmd_context *cmd, uint64_t size,
 			   uint32_t extent_size);
 
+int update_pool_lv(struct logical_volume *lv, int activate);
+int get_pool_discards(const char *str, thin_discards_t *discards);
+const char *get_pool_discards_name(thin_discards_t discards);
+
 /*
  * Activation options
  */
-typedef enum {
-	CHANGE_AY = 0,
-	CHANGE_AN = 1,
-	CHANGE_AE = 2,
-	CHANGE_ALY = 3,
-	CHANGE_ALN = 4
+typedef enum activation_change {
+	CHANGE_AY = 0,  /* activate */
+	CHANGE_AN = 1,  /* deactivate */
+	CHANGE_AE = 2,  /* activate exclusively */
+	CHANGE_ALY = 3, /* activate locally */
+	CHANGE_ALN = 4, /* deactivate locally */
+	CHANGE_AAY = 5  /* automatic activation */
 } activation_change_t;
 
 /* FIXME: refactor and reduce the size of this struct! */
 struct lvcreate_params {
 	/* flags */
 	int snapshot; /* snap */
+	int thin; /* thin */
+	int create_thin_pool; /* thin */
 	int zero; /* all */
 	int major; /* all */
 	int minor; /* all */
 	int log_count; /* mirror */
 	int nosync; /* mirror */
-	int activation_monitoring; /* all */
 	activation_change_t activate; /* non-snapshot, non-mirror */
+	thin_discards_t discards;     /* thin */
 
-	char *origin; /* snap */
+	const char *origin; /* snap */
+	const char *pool;   /* thin */
 	const char *vg_name; /* all */
 	const char *lv_name; /* all */
 
@@ -565,6 +614,8 @@ struct lvcreate_params {
 	uint32_t extents; /* all */
 	uint32_t voriginextents; /* snapshot */
 	uint64_t voriginsize; /* snapshot */
+	uint32_t poolmetadataextents; /* thin pool */
+	uint64_t poolmetadatasize; /* thin pool */
 	struct dm_list *pvh; /* all */
 
 	uint32_t permission; /* all */
@@ -625,6 +676,7 @@ const char *find_vgname_from_pvid(struct cmd_context *cmd,
 				  const char *pvid);
 /* Find LV segment containing given LE */
 struct lv_segment *first_seg(const struct logical_volume *lv);
+struct lv_segment *last_seg(const struct logical_volume *lv);
 
 
 /*
@@ -687,10 +739,9 @@ int lv_remove_mirrors(struct cmd_context *cmd, struct logical_volume *lv,
 
 int is_temporary_mirror_layer(const struct logical_volume *lv);
 struct logical_volume * find_temporary_mirror(const struct logical_volume *lv);
-int lv_is_mirrored(const struct logical_volume *lv);
 uint32_t lv_mirror_count(const struct logical_volume *lv);
 uint32_t adjusted_mirror_region_size(uint32_t extent_size, uint32_t extents,
-                                    uint32_t region_size);
+				    uint32_t region_size);
 int remove_mirrors_from_segments(struct logical_volume *lv,
 				 uint32_t new_mirrors, uint64_t status_mask);
 int add_mirrors_to_segments(struct cmd_context *cmd, struct logical_volume *lv,
@@ -712,8 +763,11 @@ int add_mirror_log(struct cmd_context *cmd, struct logical_volume *lv,
 		   uint32_t log_count, uint32_t region_size,
 		   struct dm_list *allocatable_pvs, alloc_policy_t alloc);
 
+#if 0
+/* FIXME: reconfigure_mirror_images: remove this code? */
 int reconfigure_mirror_images(struct lv_segment *mirrored_seg, uint32_t num_mirrors,
 			      struct dm_list *removable_pvs, unsigned remove_log);
+#endif
 int collapse_mirrored_lv(struct logical_volume *lv);
 int shift_mirror_images(struct lv_segment *mirrored_seg, unsigned mimage);
 
@@ -733,12 +787,12 @@ int lv_is_active_replicator_dev(const struct logical_volume *lv);
 int lv_is_replicator(const struct logical_volume *lv);
 int lv_is_replicator_dev(const struct logical_volume *lv);
 int lv_is_rimage(const struct logical_volume *lv);
-int lv_is_rlog(const struct logical_volume *lv);
 int lv_is_slog(const struct logical_volume *lv);
 struct logical_volume *first_replicator_dev(const struct logical_volume *lv);
 /* --  metadata/replicator_manip.c */
 
 /* ++  metadata/raid_manip.c */
+int lv_is_raid_with_tracking(const struct logical_volume *lv);
 uint32_t lv_raid_image_count(const struct logical_volume *lv);
 int lv_raid_change_image_count(struct logical_volume *lv,
 			       uint32_t new_count, struct dm_list *pvs);
@@ -747,6 +801,10 @@ int lv_raid_split(struct logical_volume *lv, const char *split_name,
 int lv_raid_split_and_track(struct logical_volume *lv,
 			    struct dm_list *splittable_pvs);
 int lv_raid_merge(struct logical_volume *lv);
+int lv_raid_reshape(struct logical_volume *lv,
+		    const struct segment_type *new_segtype);
+int lv_raid_replace(struct logical_volume *lv, struct dm_list *remove_pvs,
+		    struct dm_list *allocate_pvs);
 
 /* --  metadata/raid_manip.c */
 
@@ -764,12 +822,12 @@ int lv_read_replicator_vgs(struct logical_volume *lv);
 void lv_release_replicator_vgs(struct logical_volume *lv);
 
 struct logical_volume *find_pvmove_lv(struct volume_group *vg,
-				      struct device *dev, uint32_t lv_type);
+				      struct device *dev, uint64_t lv_type);
 struct logical_volume *find_pvmove_lv_from_pvname(struct cmd_context *cmd,
 						  struct volume_group *vg,
 						  const char *name,
 						  const char *uuid,
-						  uint32_t lv_type);
+						  uint64_t lv_type);
 struct logical_volume *find_pvmove_lv_in_lv(struct logical_volume *lv);
 const char *get_pvmove_pvname_from_lv(struct logical_volume *lv);
 const char *get_pvmove_pvname_from_lv_mirr(struct logical_volume *lv_mirr);
