@@ -14,10 +14,13 @@
 #ifndef _LIB_LVM2APP_H
 #define _LIB_LVM2APP_H
 
-#include "libdevmapper.h"
+#include <libdevmapper.h>
 
 #include <stdint.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /******************************** WARNING ***********************************
  *
@@ -92,6 +95,8 @@ struct lvm;
 struct physical_volume;
 struct volume_group;
 struct logical_volume;
+struct lv_segment;
+struct pv_segment;
 
 /**
  * \class lvm_t
@@ -99,7 +104,7 @@ struct logical_volume;
  * This is the base handle that is needed to open and create objects such as
  * volume groups and logical volumes.  In addition, this handle provides a
  * context for error handling information, saving any error number (see
- * lvm_errno) and error message (see lvm_errmsg) that any function may
+ * lvm_errno()) and error message (see lvm_errmsg()) that any function may
  * generate.
  */
 typedef struct lvm *lvm_t;
@@ -133,16 +138,40 @@ typedef struct logical_volume *lv_t;
  */
 typedef struct physical_volume *pv_t;
 
+/**
+ * \class lvseg_t
+ *
+ * This lv segment object is bound to a lv_t.
+ */
+typedef struct lv_segment *lvseg_t;
+
+/**
+ * \class pvseg_t
+ *
+ * This pv segment object is bound to a pv_t.
+ */
+typedef struct pv_segment *pvseg_t;
+
 /**
  * Logical Volume object list.
  *
- * Lists of these structures are returned by lvm_vg_list_pvs().
+ * Lists of these structures are returned by lvm_vg_list_lvs().
  */
 typedef struct lvm_lv_list {
 	struct dm_list list;
 	lv_t lv;
 } lv_list_t;
 
+/**
+ * Logical Volume Segment object list.
+ *
+ * Lists of these structures are returned by lvm_lv_list_lvsegs().
+ */
+typedef struct lvm_lvseg_list {
+	struct dm_list list;
+	lvseg_t lvseg;
+} lvseg_list_t;
+
 /**
  * Physical volume object list.
  *
@@ -153,6 +182,16 @@ typedef struct lvm_pv_list {
 	pv_t pv;
 } pv_list_t;
 
+/**
+ * Physical Volume Segment object list.
+ *
+ * Lists of these structures are returned by lvm_pv_list_pvsegs().
+ */
+typedef struct lvm_pvseg_list {
+	struct dm_list list;
+	pvseg_t pvseg;
+} pvseg_list_t;
+
 /**
  * String list.
  *
@@ -165,13 +204,37 @@ typedef struct lvm_str_list {
 	const char *str;
 } lvm_str_list_t;
 
+/**
+ * Property Value
+ *
+ * This structure defines a single LVM property value for an LVM object.
+ * The structures are returned by functions such as
+ * lvm_vg_get_property().
+ *
+ * is_settable: indicates whether a 'set' function exists for this property
+ * is_string: indicates whether this property is a string (1) or not (0)
+ * is_integer: indicates whether this property is an integer (1) or not (0)
+ * is_valid: indicates whether 'value' is valid (1) or not (0)
+ */
+typedef struct lvm_property_value {
+	uint32_t is_settable:1;
+	uint32_t is_string:1;
+	uint32_t is_integer:1;
+	uint32_t is_valid:1;
+	uint32_t padding:28;
+	union {
+		const char *string;
+		uint64_t integer;
+	} value;
+} lvm_property_value_t;
+
 /*************************** generic lvm handling ***************************/
 /**
  * Create a LVM handle.
  *
  * \memberof lvm_t
  *
- * Once all LVM operations have been completed, use lvm_quit to release
+ * Once all LVM operations have been completed, use lvm_quit() to release
  * the handle and any associated resources.
  *
  * \param system_dir
@@ -182,7 +245,7 @@ typedef struct lvm_str_list {
  * \return
  * A valid LVM handle is returned or NULL if there has been a
  * memory allocation problem. You have to check if an error occured
- * with the lvm_error function.
+ * with the lvm_error() function.
  */
 lvm_t lvm_init(const char *system_dir);
 
@@ -239,6 +302,28 @@ int lvm_config_reload(lvm_t libh);
  */
 int lvm_config_override(lvm_t libh, const char *config_string);
 
+/**
+ * Find a boolean value in the LVM configuration.
+ *
+ * \memberof lvm_t
+ *
+ * This function finds a boolean value associated with a path
+ * in current LVM configuration.
+ *
+ * \param   libh
+ * Handle obtained from lvm_init().
+ *
+ * \param   config_path
+ * A path in LVM configuration
+ *
+ * \param   fail
+ * Value to return if the path is not found.
+ *
+ * \return
+ * boolean value for 'config_path' (success) or the value of 'fail' (error)
+ */
+int lvm_config_find_bool(lvm_t libh, const char *config_path, int fail);
+
 /**
  * Return stored error no describing last LVM API error.
  *
@@ -265,7 +350,7 @@ int lvm_errno(lvm_t libh);
  *
  * \memberof lvm_t
  *
- * This function may be used in conjunction with lvm_errno to obtain more
+ * This function may be used in conjunction with lvm_errno() to obtain more
  * specific error information for a function that is known to have failed.
  *
  * \param   libh
@@ -292,10 +377,10 @@ int lvm_scan(lvm_t libh);
  * \memberof lvm_t
  *
  * The memory allocated for the list is tied to the lvm_t handle and will be
- * released when lvm_quit is called.
+ * released when lvm_quit() is called.
  *
  * NOTE: This function normally does not scan devices in the system for LVM
- * metadata.  To scan the system, use lvm_scan.
+ * metadata.  To scan the system, use lvm_scan().
  *
  * To process the list, use the dm_list iterator functions.  For example:
  *      vg_t vg;
@@ -326,10 +411,10 @@ struct dm_list *lvm_list_vg_names(lvm_t libh);
  * \memberof lvm_t
  *
  * The memory allocated for the list is tied to the lvm_t handle and will be
- * released when lvm_quit is called.
+ * released when lvm_quit() is called.
  *
  * NOTE: This function normally does not scan devices in the system for LVM
- * metadata.  To scan the system, use lvm_scan.
+ * metadata.  To scan the system, use lvm_scan().
  *
  * \param   libh
  * Handle obtained from lvm_init().
@@ -349,7 +434,7 @@ struct dm_list *lvm_list_vg_uuids(lvm_t libh);
  * \memberof lvm_t
  *
  * The memory allocated for the name is tied to the lvm_t handle and will be
- * released when lvm_quit is called.
+ * released when lvm_quit() is called.
  *
  * NOTE: This function may scan devices in the system for LVM metadata.
  *
@@ -368,7 +453,7 @@ const char *lvm_vgname_from_pvid(lvm_t libh, const char *pvid);
  * \memberof lvm_t
  *
  * The memory allocated for the name is tied to the lvm_t handle and will be
- * released when lvm_quit is called.
+ * released when lvm_quit() is called.
  *
  * NOTE: This function may scan devices in the system for LVM metadata.
  *
@@ -440,7 +525,7 @@ vg_t lvm_vg_create(lvm_t libh, const char *vg_name);
  * \memberof vg_t
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * A list of lvm_lv_list structures containing lv handles for this vg.
@@ -454,7 +539,7 @@ struct dm_list *lvm_vg_list_lvs(vg_t vg);
  * \memberof vg_t
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * A list of lvm_pv_list structures containing pv handles for this vg.
@@ -472,7 +557,7 @@ struct dm_list *lvm_vg_list_pvs(vg_t vg);
  * with lvm_vg_close().
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * 0 (success) or -1 (failure).
@@ -488,7 +573,7 @@ int lvm_vg_write(vg_t vg);
  * calling lvm_vg_write() to commit the removal to disk.
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * 0 (success) or -1 (failure).
@@ -504,7 +589,7 @@ int lvm_vg_remove(vg_t vg);
  * handle.
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * 0 (success) or -1 (failure).
@@ -523,10 +608,10 @@ int lvm_vg_close(vg_t vg);
  * If the device is not initialized for LVM use, it will be initialized
  * before adding to the VG.  Although some internal checks are done,
  * the caller should be sure the device is not in use by other subsystems
- * before calling lvm_vg_extend.
+ * before calling lvm_vg_extend().
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \param   device
  * Absolute pathname of device to add to VG.
@@ -547,7 +632,7 @@ int lvm_vg_extend(vg_t vg, const char *device);
  * lvm_vg_close().
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \param   device
  * Name of device to remove from VG.
@@ -568,7 +653,7 @@ int lvm_vg_reduce(vg_t vg, const char *device);
  * with lvm_vg_close().
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \param   tag
  * Tag to add to the VG.
@@ -589,7 +674,7 @@ int lvm_vg_add_tag(vg_t vg, const char *tag);
  * with lvm_vg_close().
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \param   tag
  * Tag to remove from VG.
@@ -610,7 +695,7 @@ int lvm_vg_remove_tag(vg_t vg, const char *tag);
  * handle with lvm_vg_close().
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \param   new_size
  * New extent size in bytes.
@@ -626,7 +711,7 @@ int lvm_vg_set_extent_size(vg_t vg, uint32_t new_size);
  * \memberof vg_t
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * 1 if the VG is clustered, 0 if not
@@ -639,7 +724,7 @@ uint64_t lvm_vg_is_clustered(vg_t vg);
  * \memberof vg_t
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * 1 if the VG is exported, 0 if not
@@ -656,7 +741,7 @@ uint64_t lvm_vg_is_exported(vg_t vg);
  * group.
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * 1 if the VG is PVs, 0 if not
@@ -673,7 +758,7 @@ uint64_t lvm_vg_is_partial(vg_t vg);
  * have changed from a prior query.
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * Metadata sequence number.
@@ -689,7 +774,7 @@ uint64_t lvm_vg_get_seqno(const vg_t vg);
  * released when lvm_vg_close() is called.
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * Copy of the uuid string.
@@ -705,7 +790,7 @@ const char *lvm_vg_get_uuid(const vg_t vg);
  * released when lvm_vg_close() is called.
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * Copy of the name.
@@ -718,7 +803,7 @@ const char *lvm_vg_get_name(const vg_t vg);
  * \memberof vg_t
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * Size in bytes.
@@ -731,7 +816,7 @@ uint64_t lvm_vg_get_size(const vg_t vg);
  * \memberof vg_t
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * Free size in bytes.
@@ -744,7 +829,7 @@ uint64_t lvm_vg_get_free_size(const vg_t vg);
  * \memberof vg_t
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * Extent size in bytes.
@@ -757,7 +842,7 @@ uint64_t lvm_vg_get_extent_size(const vg_t vg);
  * \memberof vg_t
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * Extent count.
@@ -770,7 +855,7 @@ uint64_t lvm_vg_get_extent_count(const vg_t vg);
  * \memberof vg_t
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * Free extent count.
@@ -783,7 +868,7 @@ uint64_t lvm_vg_get_free_extent_count(const vg_t vg);
  * \memberof vg_t
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * Physical volume count.
@@ -796,7 +881,7 @@ uint64_t lvm_vg_get_pv_count(const vg_t vg);
  * \memberof vg_t
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * Maximum number of physical volumes allowed in a volume group.
@@ -809,7 +894,7 @@ uint64_t lvm_vg_get_max_pv(const vg_t vg);
  * \memberof vg_t
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \return
  * Maximum number of logical volumes allowed in a volume group.
@@ -845,6 +930,75 @@ uint64_t lvm_vg_get_max_lv(const vg_t vg);
  */
 struct dm_list *lvm_vg_get_tags(const vg_t vg);
 
+/**
+ * Get the value of a VG property
+ *
+ * \memberof vg_t
+ *
+ * \param   vg
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
+ *
+ * \param   name
+ * Name of property to query.  See vgs man page for full list of properties
+ * that may be queried.
+ *
+ * The memory allocated for a string property value is tied to the vg_t
+ * handle and will be released when lvm_vg_close() is called.
+ *
+ * Example:
+ *      lvm_property_value v;
+ *      char *prop_name = "vg_mda_count";
+ *
+ *      v = lvm_vg_get_property(vg, prop_name);
+ *      if (!v.is_valid) {
+ *           printf("Invalid property name or unable to query"
+ *                  "'%s', errno = %d.\n", prop_name, lvm_errno(libh));
+ *           return;
+ *      }
+ *      if (v.is_string)
+ *           printf(", value = %s\n", v.value.string);
+ *	if (v.is_integer)
+ *           printf(", value = %"PRIu64"\n", v.value.integer);
+ *
+ *
+ * \return
+ * lvm_property_value structure that will contain the current
+ * value of the property.  Caller should check 'is_valid' flag before using
+ * the value.  If 'is_valid' is not set, caller should check lvm_errno()
+ * for specific error.
+ */
+struct lvm_property_value lvm_vg_get_property(const vg_t vg, const char *name);
+
+/**
+ * Set the value of a VG property.  Note that the property must be
+ * a 'settable' property, as evidenced by the 'is_settable' flag
+ * when querying the property.
+ *
+ * \memberof vg_t
+ *
+ * The memory allocated for a string property value is tied to the vg_t
+ * handle and will be released when lvm_vg_close() is called.
+ *
+ * Example (integer):
+ *      lvm_property_value copies;
+ *
+ *      if (lvm_vg_get_property(vg, "vg_mda_copies", &copies) < 0) {
+ *              // Error - unable to query property
+ *      }
+ *      if (!copies.is_settable) {
+ *              // Error - property not settable
+ *      }
+ *      copies.value.integer = 2;
+ *      if (lvm_vg_set_property(vg, "vg_mda_copies", &copies) < 0) {
+ *              // handle error
+ *      }
+ *
+ * \return
+ * 0 (success) or -1 (failure).
+ */
+int lvm_vg_set_property(const vg_t vg, const char *name,
+			struct lvm_property_value *value);
+
 /************************** logical volume handling *************************/
 
 /**
@@ -855,7 +1009,7 @@ struct dm_list *lvm_vg_get_tags(const vg_t vg);
  * as the API is developed.
  *
  * \param   vg
- * VG handle obtained from lvm_vg_create or lvm_vg_open().
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
  *
  * \param   name
  * Name of logical volume to create.
@@ -869,6 +1023,55 @@ struct dm_list *lvm_vg_get_tags(const vg_t vg);
  */
 lv_t lvm_vg_create_lv_linear(vg_t vg, const char *name, uint64_t size);
 
+/**
+ * Return a list of lvseg handles for a given LV handle.
+ *
+ * \memberof lv_t
+ *
+ * \param   lv
+ * Logical volume handle.
+ *
+ * \return
+ * A list of lvm_lvseg_list structures containing lvseg handles for this lv.
+ */
+struct dm_list *lvm_lv_list_lvsegs(lv_t lv);
+
+/**
+ * Lookup an LV handle in a VG by the LV name.
+ *
+ * \memberof lv_t
+ *
+ * \param   vg
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
+ *
+ * \param   name
+ * Name of LV to lookup.
+ *
+ * \return
+ * non-NULL handle to the LV 'name' attached to the VG.
+ * NULL is returned if the LV name is not associated with the VG handle.
+ */
+lv_t lvm_lv_from_name(vg_t vg, const char *name);
+
+/**
+ * Lookup an LV handle in a VG by the LV uuid.
+ * The form of the uuid may be either the formatted, human-readable form,
+ * or the non-formatted form.
+ *
+ * \memberof lv_t
+ *
+ * \param   vg
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
+ *
+ * \param   uuid
+ * UUID of LV to lookup.
+ *
+ * \return
+ * non-NULL handle to the LV with 'uuid' attached to the VG.
+ * NULL is returned if the LV uuid is not associated with the VG handle.
+ */
+lv_t lvm_lv_from_uuid(vg_t vg, const char *uuid);
+
 /**
  * Activate a logical volume.
  *
@@ -926,8 +1129,8 @@ int lvm_vg_remove_lv(lv_t lv);
  *
  * \memberof lv_t
  *
- * Memory is allocated using dm_malloc() and caller must free the memory
- * using dm_free().
+ * The memory allocated for the uuid is tied to the vg_t handle and will be
+ * released when lvm_vg_close() is called.
  *
  * \param   lv
  * Logical volume handle.
@@ -942,8 +1145,8 @@ const char *lvm_lv_get_uuid(const lv_t lv);
  *
  * \memberof lv_t
  *
- * Memory is allocated using dm_malloc() and caller must free the memory
- * using dm_free().
+ * The memory allocated for the name is tied to the vg_t handle and will be
+ * released when lvm_vg_close() is called.
  *
  * \param   lv
  * Logical volume handle.
@@ -966,6 +1169,83 @@ const char *lvm_lv_get_name(const lv_t lv);
  */
 uint64_t lvm_lv_get_size(const lv_t lv);
 
+/**
+ * Get the value of a LV property
+ *
+ * \memberof lv_t
+ *
+ * \param   lv
+ * Logical volume handle.
+ *
+ * \param   name
+ * Name of property to query.  See lvs man page for full list of properties
+ * that may be queried.
+ *
+ * The memory allocated for a string property value is tied to the vg_t
+ * handle and will be released when lvm_vg_close() is called.
+ *
+ * Example:
+ *      lvm_property_value v;
+ *      char *prop_name = "seg_count";
+ *
+ *      v = lvm_lv_get_property(lv, prop_name);
+ *      if (!v.is_valid) {
+ *           printf("Invalid property name or unable to query"
+ *                  "'%s', errno = %d.\n", prop_name, lvm_errno(libh));
+ *           return;
+ *      }
+ *      if (v.is_string)
+ *           printf(", value = %s\n", v.value.string);
+ *	if (v.is_integer)
+ *           printf(", value = %"PRIu64"\n", v.value.integer);
+ *
+ * \return
+ * lvm_property_value structure that will contain the current
+ * value of the property.  Caller should check 'is_valid' flag before using
+ * the value.  If 'is_valid' is not set, caller should check lvm_errno()
+ * for specific error.
+ */
+struct lvm_property_value lvm_lv_get_property(const lv_t lv, const char *name);
+
+/**
+ * Get the value of a LV segment property
+ *
+ * \memberof lv_t
+ *
+ * \param   lvseg
+ * Logical volume segment handle.
+ *
+ * \param   name
+ * Name of property to query.  See lvs man page for full list of properties
+ * that may be queried.
+ *
+ * The memory allocated for a string property value is tied to the vg_t
+ * handle and will be released when lvm_vg_close() is called.
+ *
+ * Example:
+ *      lvm_property_value v;
+ *      char *prop_name = "seg_start_pe";
+ *
+ *      v = lvm_lvseg_get_property(lvseg, prop_name);
+ *      if (lvm_errno(libh) || !v.is_valid) {
+ *           // handle error
+ *           printf("Invalid property name or unable to query"
+ *                  "'%s'.\n", prop_name);
+ *           return;
+ *      }
+ *      if (v.is_string)
+ *           printf(", value = %s\n", v.value.string);
+ *	else
+ *           printf(", value = %"PRIu64"\n", v.value.integer);
+ *
+ * \return
+ * lvm_property_value structure that will contain the current
+ * value of the property.  Caller should check lvm_errno() as well
+ * as 'is_valid' flag before using the value.
+ */
+struct lvm_property_value lvm_lvseg_get_property(const lvseg_t lvseg,
+						 const char *name);
+
 /**
  * Get the current activation state of a logical volume.
  *
@@ -1063,6 +1343,22 @@ int lvm_lv_remove_tag(lv_t lv, const char *tag);
  */
 struct dm_list *lvm_lv_get_tags(const lv_t lv);
 
+/**
+ * Rename logical volume to new_name.
+ *
+ * \memberof lv_t
+ *
+ * \param   lv
+ * Logical volume handle.
+ *
+ * \param   new_name
+ * New name of logical volume.
+ *
+ * \return
+ * 0 (success) or -1 (failure).
+ *
+ */
+int lvm_lv_rename(lv_t lv, const char *new_name);
 
 /**
  * Resize logical volume to new_size bytes.
@@ -1112,7 +1408,7 @@ const char *lvm_pv_get_uuid(const pv_t pv);
  *
  * \memberof pv_t
  *
- * The memory allocated for the uuid is tied to the vg_t handle and will be
+ * The memory allocated for the name is tied to the vg_t handle and will be
  * released when lvm_vg_close() is called.
  *
  * \param   pv
@@ -1176,6 +1472,134 @@ uint64_t lvm_pv_get_size(const pv_t pv);
  */
 uint64_t lvm_pv_get_free(const pv_t pv);
 
+/**
+ * Get the value of a PV property
+ *
+ * \memberof pv_t
+ *
+ * \param   pv
+ * Physical volume handle.
+ *
+ * \param   name
+ * Name of property to query.  See pvs man page for full list of properties
+ * that may be queried.
+ *
+ * The memory allocated for a string property value is tied to the vg_t handle
+ * and will be released when lvm_vg_close() is called. For "percent" values
+ * (those obtained for copy_percent and snap_percent properties), please see
+ * percent_range_t and lvm_percent_to_float().
+ *
+ * Example:
+ *      lvm_property_value value;
+ *      char *prop_name = "pv_mda_count";
+ *
+ *      v = lvm_pv_get_property(pv, prop_name);
+ *      if (!v.is_valid) {
+ *           printf("Invalid property name or unable to query"
+ *                  "'%s', errno = %d.\n", prop_name, lvm_errno(libh));
+ *           return;
+ *      }
+ *      if (v.is_string)
+ *           printf(", value = %s\n", v.value.string);
+ *	if (v.is_integer)
+ *           printf(", value = %"PRIu64"\n", v.value.integer);
+ *
+ * \return
+ * lvm_property_value structure that will contain the current
+ * value of the property.  Caller should check 'is_valid' flag before using
+ * the value.  If 'is_valid' is not set, caller should check lvm_errno()
+ * for specific error.
+ */
+struct lvm_property_value lvm_pv_get_property(const pv_t pv, const char *name);
+
+/**
+ * Get the value of a PV segment property
+ *
+ * \memberof pv_t
+ *
+ * \param   pvseg
+ * Physical volume segment handle.
+ *
+ * \param   name
+ * Name of property to query.  See pvs man page for full list of properties
+ * that may be queried.
+ *
+ * The memory allocated for a string property value is tied to the vg_t
+ * handle and will be released when lvm_vg_close() is called.
+ *
+ * Example:
+ *      lvm_property_value v;
+ *      char *prop_name = "pvseg_start";
+ *
+ *      v = lvm_pvseg_get_property(pvseg, prop_name);
+ *      if (lvm_errno(libh) || !v.is_valid) {
+ *           // handle error
+ *           printf("Invalid property name or unable to query"
+ *                  "'%s'.\n", prop_name);
+ *           return;
+ *      }
+ *      if (v.is_string)
+ *           printf(", value = %s\n", v.value.string);
+ *	else
+ *           printf(", value = %"PRIu64"\n", v.value.integer);
+ *
+ * \return
+ * lvm_property_value structure that will contain the current
+ * value of the property.  Caller should check lvm_errno() as well
+ * as 'is_valid' flag before using the value.
+ */
+struct lvm_property_value lvm_pvseg_get_property(const pvseg_t pvseg,
+						 const char *name);
+
+/**
+ * Return a list of pvseg handles for a given PV handle.
+ *
+ * \memberof pv_t
+ *
+ * \param   pv
+ * Physical volume handle.
+ *
+ * \return
+ * A list of lvm_pvseg_list structures containing pvseg handles for this pv.
+ */
+struct dm_list *lvm_pv_list_pvsegs(pv_t pv);
+
+/**
+ * Lookup an PV handle in a VG by the PV name.
+ *
+ * \memberof pv_t
+ *
+ * \param   vg
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
+ *
+ * \param   name
+ * Name of PV to lookup.
+ *
+ * \return
+ * non-NULL handle to the PV 'name' attached to the VG.
+ * NULL is returned if the PV name is not associated with the VG handle.
+ */
+pv_t lvm_pv_from_name(vg_t vg, const char *name);
+
+/**
+ * Lookup an PV handle in a VG by the PV uuid.
+ * The form of the uuid may be either the formatted, human-readable form,
+ * or the non-formatted form.
+ *
+ * \memberof pv_t
+ *
+ * \param   vg
+ * VG handle obtained from lvm_vg_create() or lvm_vg_open().
+ *
+ * \param   uuid
+ * UUID of PV to lookup.
+ *
+ * \return
+ * non-NULL handle to the PV with 'uuid' attached to the VG.
+ * NULL is returned if the PV uuid is not associated with the VG handle.
+ */
+pv_t lvm_pv_from_uuid(vg_t vg, const char *uuid);
+
 /**
  * Resize physical volume to new_size bytes.
  *
@@ -1194,4 +1618,32 @@ uint64_t lvm_pv_get_free(const pv_t pv);
  */
 int lvm_pv_resize(const pv_t pv, uint64_t new_size);
 
+#ifndef _LVM_PERCENT_H
+
+/**
+ * This type defines a couple of special percent values. The PERCENT_0 and
+ * PERCENT_100 constants designate *exact* percentages: values are never
+ * rounded to either of these two.
+ */
+typedef enum {
+	PERCENT_0 = 0,
+	PERCENT_1 = 1000000,
+	PERCENT_100 = 100 * PERCENT_1,
+	PERCENT_INVALID = -1,
+	PERCENT_MERGE_FAILED = -2
+} percent_range_t;
+
+typedef int32_t percent_t;
+
+#endif
+
+/**
+ * Convert a (fixed-point) value obtained from the percent-denominated
+ * *_get_property functions into a floating-point value.
+ */
+float lvm_percent_to_float(percent_t v);
+
+#ifdef __cplusplus
+}
+#endif
 #endif /* _LIB_LVM2APP_H */
